@@ -2,6 +2,7 @@
 
 import gc
 import sys
+from typing import Callable
 from unittest import mock
 
 import pytest
@@ -15,6 +16,17 @@ from aiohttp.helpers import TimerNoop
 from aiohttp.test_utils import make_mocked_coro
 
 
+class WriterMock(mock.AsyncMock):
+    def __await__(self) -> None:
+        return self().__await__()
+
+    def add_done_callback(self, cb: Callable[[], None]) -> None:
+        cb()
+
+    def done(self) -> bool:
+        return True
+
+
 @pytest.fixture
 def session():
     return mock.Mock()
@@ -27,7 +39,7 @@ async def test_http_processing_error(session) -> None:
         "get",
         URL("http://del-cl-resp.org"),
         request_info=request_info,
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -55,7 +67,7 @@ def test_del(session) -> None:
         "get",
         URL("http://del-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -82,7 +94,7 @@ def test_close(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -103,7 +115,7 @@ def test_wait_for_100_1(loop, session) -> None:
         URL("http://python.org"),
         continue100=object(),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         timer=TimerNoop(),
         traces=[],
         loop=loop,
@@ -119,7 +131,7 @@ def test_wait_for_100_2(loop, session) -> None:
         URL("http://python.org"),
         request_info=mock.Mock(),
         continue100=None,
-        writer=mock.Mock(),
+        writer=WriterMock(),
         timer=TimerNoop(),
         traces=[],
         loop=loop,
@@ -134,7 +146,7 @@ def test_repr(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -151,7 +163,7 @@ def test_repr_non_ascii_url() -> None:
         "get",
         URL("http://fake-host.org/\u03bb"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -166,7 +178,7 @@ def test_repr_non_ascii_reason() -> None:
         "get",
         URL("http://fake-host.org/path"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -200,7 +212,7 @@ async def test_read_and_release_connection(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -226,7 +238,7 @@ async def test_read_and_release_connection_with_error(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -247,7 +259,7 @@ async def test_release(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -276,7 +288,7 @@ async def test_release_on_del(loop, session) -> None:
             "get",
             URL("http://def-cl-resp.org"),
             request_info=mock.Mock(),
-            writer=mock.Mock(),
+            writer=WriterMock(),
             continue100=None,
             timer=TimerNoop(),
             traces=[],
@@ -296,7 +308,7 @@ async def test_response_eof(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=None,
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -317,7 +329,7 @@ async def test_response_eof_upgraded(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -338,7 +350,7 @@ async def test_response_eof_after_connection_detach(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=None,
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -359,7 +371,7 @@ async def test_text(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -386,7 +398,7 @@ async def test_text_bad_encoding(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -416,7 +428,7 @@ async def test_text_custom_encoding(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -440,12 +452,14 @@ async def test_text_custom_encoding(loop, session) -> None:
     assert not response.get_encoding.called
 
 
-async def test_text_detect_encoding(loop, session) -> None:
+@pytest.mark.parametrize("content_type", ("text/plain", "text/plain;charset=invalid"))
+async def test_text_charset_resolver(content_type: str, loop, session) -> None:
+    session._resolve_charset = lambda r, b: "cp1251"
     response = ClientResponse(
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -458,7 +472,7 @@ async def test_text_detect_encoding(loop, session) -> None:
         fut.set_result('{"тест": "пройден"}'.encode("cp1251"))
         return fut
 
-    response._headers = {"Content-Type": "text/plain"}
+    response._headers = {"Content-Type": content_type}
     content = response.content = mock.Mock()
     content.read.side_effect = side_effect
 
@@ -466,35 +480,7 @@ async def test_text_detect_encoding(loop, session) -> None:
     res = await response.text()
     assert res == '{"тест": "пройден"}'
     assert response._connection is None
-
-
-async def test_text_detect_encoding_if_invalid_charset(loop, session) -> None:
-    response = ClientResponse(
-        "get",
-        URL("http://def-cl-resp.org"),
-        request_info=mock.Mock(),
-        writer=mock.Mock(),
-        continue100=None,
-        timer=TimerNoop(),
-        traces=[],
-        loop=loop,
-        session=session,
-    )
-
-    def side_effect(*args, **kwargs):
-        fut = loop.create_future()
-        fut.set_result('{"тест": "пройден"}'.encode("cp1251"))
-        return fut
-
-    response._headers = {"Content-Type": "text/plain;charset=invalid"}
-    content = response.content = mock.Mock()
-    content.read.side_effect = side_effect
-
-    await response.read()
-    res = await response.text()
-    assert res == '{"тест": "пройден"}'
-    assert response._connection is None
-    assert response.get_encoding().lower() in ("windows-1251", "maccyrillic")
+    assert response.get_encoding() == "cp1251"
 
 
 async def test_get_encoding_body_none(loop, session) -> None:
@@ -502,7 +488,7 @@ async def test_get_encoding_body_none(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -521,7 +507,7 @@ async def test_get_encoding_body_none(loop, session) -> None:
 
     with pytest.raises(
         RuntimeError,
-        match="^Cannot guess the encoding of a not yet read body$",
+        match="^Cannot compute fallback encoding of a not yet read body$",
     ):
         response.get_encoding()
     assert response.closed
@@ -532,7 +518,7 @@ async def test_text_after_read(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -559,7 +545,7 @@ async def test_json(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -586,7 +572,7 @@ async def test_json_extended_content_type(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -615,7 +601,7 @@ async def test_json_custom_content_type(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -642,7 +628,7 @@ async def test_json_custom_loader(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -664,7 +650,7 @@ async def test_json_invalid_content_type(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -685,7 +671,7 @@ async def test_json_no_content(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -704,7 +690,7 @@ async def test_json_override_encoding(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -733,7 +719,7 @@ def test_get_encoding_unknown(loop, session) -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -742,9 +728,7 @@ def test_get_encoding_unknown(loop, session) -> None:
     )
 
     response._headers = {"Content-Type": "application/json"}
-    with mock.patch("aiohttp.client_reqrep.chardet") as m_chardet:
-        m_chardet.detect.return_value = {"encoding": None}
-        assert response.get_encoding() == "utf-8"
+    assert response.get_encoding() == "utf-8"
 
 
 def test_raise_for_status_2xx() -> None:
@@ -752,7 +736,7 @@ def test_raise_for_status_2xx() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -769,7 +753,7 @@ def test_raise_for_status_4xx() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -790,7 +774,7 @@ def test_raise_for_status_4xx_without_reason() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -811,7 +795,7 @@ def test_resp_host() -> None:
         "get",
         URL("http://del-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -826,7 +810,7 @@ def test_content_type() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -843,7 +827,7 @@ def test_content_type_no_header() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -860,7 +844,7 @@ def test_charset() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -877,7 +861,7 @@ def test_charset_no_header() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -894,7 +878,7 @@ def test_charset_no_charset() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -911,7 +895,7 @@ def test_content_disposition_full() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -934,7 +918,7 @@ def test_content_disposition_no_parameters() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -953,7 +937,7 @@ def test_content_disposition_no_header() -> None:
         "get",
         URL("http://def-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -972,7 +956,7 @@ def test_response_request_info() -> None:
         "get",
         URL(url),
         request_info=RequestInfo(url, "get", headers),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -991,7 +975,7 @@ def test_request_info_in_exception() -> None:
         "get",
         URL(url),
         request_info=RequestInfo(url, "get", headers),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1012,7 +996,7 @@ def test_no_redirect_history_in_exception() -> None:
         "get",
         URL(url),
         request_info=RequestInfo(url, "get", headers),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1035,7 +1019,7 @@ def test_redirect_history_in_exception() -> None:
         "get",
         URL(url),
         request_info=RequestInfo(url, "get", headers),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1049,7 +1033,7 @@ def test_redirect_history_in_exception() -> None:
         "get",
         URL(hist_url),
         request_info=RequestInfo(url, "get", headers),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1078,7 +1062,7 @@ async def test_response_read_triggers_callback(loop, session) -> None:
         response_method,
         response_url,
         request_info=mock.Mock,
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         loop=loop,
@@ -1111,7 +1095,7 @@ def test_response_real_url(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1128,7 +1112,7 @@ def test_response_links_comma_separated(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1158,7 +1142,7 @@ def test_response_links_multiple_headers(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1183,7 +1167,7 @@ def test_response_links_no_rel(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1202,7 +1186,7 @@ def test_response_links_quoted(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1225,7 +1209,7 @@ def test_response_links_relative(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1248,7 +1232,7 @@ def test_response_links_empty(loop, session) -> None:
         "get",
         url,
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
@@ -1264,7 +1248,7 @@ def test_response_not_closed_after_get_ok(mocker) -> None:
         "get",
         URL("http://del-cl-resp.org"),
         request_info=mock.Mock(),
-        writer=mock.Mock(),
+        writer=WriterMock(),
         continue100=None,
         timer=TimerNoop(),
         traces=[],
