@@ -8,19 +8,19 @@ import socket
 import ssl
 import subprocess
 import sys
+import time
+from typing import Callable, NoReturn, Set
 from unittest import mock
 from uuid import uuid4
 
 import pytest
-from conftest import IS_UNIX, needs_unix
 
-from aiohttp import web
-from aiohttp.helpers import PY_37
+from aiohttp import ClientConnectorError, ClientSession, WSCloseCode, web
 from aiohttp.test_utils import make_mocked_coro
 from aiohttp.web_runner import BaseRunner
 
 # Test for features of OS' socket support
-if IS_UNIX:
+if hasattr(socket, "AF_UNIX"):
     with socket.socket(socket.AF_UNIX, socket.SOCK_STREAM) as _abstract_path_sock:
         try:
             _abstract_path_sock.bind(b"\x00" + uuid4().hex.encode("ascii"))
@@ -36,7 +36,7 @@ else:
 skip_if_no_abstract_paths = pytest.mark.skipif(
     _abstract_path_failed, reason="Linux-style abstract paths are not supported."
 )
-del IS_UNIX, _abstract_path_failed
+del _abstract_path_failed
 
 HAS_IPV6 = socket.has_ipv6
 if HAS_IPV6:
@@ -50,12 +50,6 @@ if HAS_IPV6:
         HAS_IPV6 = False
 
 
-# tokio event loop does not allow to override attributes
-def skip_if_no_dict(loop):
-    if not hasattr(loop, "__dict__"):
-        pytest.skip("can not override loop attributes")
-
-
 def skip_if_on_windows():
     if platform.system() == "Windows":
         pytest.skip("the test is not valid for Windows")
@@ -63,7 +57,6 @@ def skip_if_on_windows():
 
 @pytest.fixture
 def patched_loop(loop):
-    skip_if_no_dict(loop)
     server = mock.Mock()
     server.wait_closed = make_mocked_coro(None)
     loop.create_server = make_mocked_coro(server)
@@ -537,7 +530,7 @@ def test_run_app_https_unix_socket(patched_loop, unix_sockname) -> None:
     assert f"https://unix:{unix_sockname}:" in printer.call_args[0][0]
 
 
-@needs_unix
+@pytest.mark.skipif(not hasattr(socket, "AF_UNIX"), reason="requires UNIX sockets")
 @skip_if_no_abstract_paths
 def test_run_app_abstract_linux_socket(patched_loop) -> None:
     sock_path = b"\x00" + uuid4().hex.encode("ascii")
@@ -559,7 +552,7 @@ def test_run_app_preexisting_inet_socket(patched_loop, mocker) -> None:
 
     sock = socket.socket()
     with contextlib.closing(sock):
-        sock.bind(("0.0.0.0", 0))
+        sock.bind(("127.0.0.1", 0))
         _, port = sock.getsockname()
 
         printer = mock.Mock(wraps=stopper(patched_loop))
@@ -568,7 +561,7 @@ def test_run_app_preexisting_inet_socket(patched_loop, mocker) -> None:
         patched_loop.create_server.assert_called_with(
             mock.ANY, sock=sock, backlog=128, ssl=None
         )
-        assert f"http://0.0.0.0:{port}" in printer.call_args[0][0]
+        assert f"http://127.0.0.1:{port}" in printer.call_args[0][0]
 
 
 @pytest.mark.skipif(not HAS_IPV6, reason="IPv6 is not available")
@@ -577,7 +570,7 @@ def test_run_app_preexisting_inet6_socket(patched_loop) -> None:
 
     sock = socket.socket(socket.AF_INET6)
     with contextlib.closing(sock):
-        sock.bind(("::", 0))
+        sock.bind(("::1", 0))
         port = sock.getsockname()[1]
 
         printer = mock.Mock(wraps=stopper(patched_loop))
@@ -586,10 +579,10 @@ def test_run_app_preexisting_inet6_socket(patched_loop) -> None:
         patched_loop.create_server.assert_called_with(
             mock.ANY, sock=sock, backlog=128, ssl=None
         )
-        assert f"http://[::]:{port}" in printer.call_args[0][0]
+        assert f"http://[::1]:{port}" in printer.call_args[0][0]
 
 
-@needs_unix
+@pytest.mark.skipif(not hasattr(socket, "AF_UNIX"), reason="requires UNIX sockets")
 def test_run_app_preexisting_unix_socket(patched_loop, mocker) -> None:
     app = web.Application()
 
@@ -614,9 +607,9 @@ def test_run_app_multiple_preexisting_sockets(patched_loop) -> None:
     sock1 = socket.socket()
     sock2 = socket.socket()
     with contextlib.closing(sock1), contextlib.closing(sock2):
-        sock1.bind(("0.0.0.0", 0))
+        sock1.bind(("localhost", 0))
         _, port1 = sock1.getsockname()
-        sock2.bind(("0.0.0.0", 0))
+        sock2.bind(("localhost", 0))
         _, port2 = sock2.getsockname()
 
         printer = mock.Mock(wraps=stopper(patched_loop))
@@ -628,8 +621,8 @@ def test_run_app_multiple_preexisting_sockets(patched_loop) -> None:
                 mock.call(mock.ANY, sock=sock2, backlog=128, ssl=None),
             ]
         )
-        assert f"http://0.0.0.0:{port1}" in printer.call_args[0][0]
-        assert f"http://0.0.0.0:{port2}" in printer.call_args[0][0]
+        assert f"http://127.0.0.1:{port1}" in printer.call_args[0][0]
+        assert f"http://127.0.0.1:{port2}" in printer.call_args[0][0]
 
 
 _script_test_signal = """
@@ -881,7 +874,6 @@ def test_run_app_keepalive_timeout(patched_loop, mocker, monkeypatch):
     )
 
 
-@pytest.mark.skipif(not PY_37, reason="contextvars support is required")
 def test_run_app_context_vars(patched_loop):
     from contextvars import ContextVar
 
@@ -912,3 +904,284 @@ def test_run_app_context_vars(patched_loop):
 
     web.run_app(init(), print=stopper(patched_loop), loop=patched_loop)
     assert count == 3
+
+
+class TestShutdown:
+    def raiser(self) -> NoReturn:
+        raise KeyboardInterrupt
+
+    async def stop(self, request: web.Request) -> web.Response:
+        asyncio.get_running_loop().call_soon(self.raiser)
+        return web.Response()
+
+    def run_app(self, port: int, timeout: int, task, extra_test=None) -> asyncio.Task:
+        async def test() -> None:
+            await asyncio.sleep(1)
+            async with ClientSession() as sess:
+                async with sess.get(f"http://localhost:{port}/"):
+                    pass
+                async with sess.get(f"http://localhost:{port}/stop"):
+                    pass
+
+                if extra_test:
+                    await extra_test(sess)
+
+        async def run_test(app: web.Application) -> None:
+            nonlocal test_task
+            test_task = asyncio.create_task(test())
+            yield
+            await test_task
+
+        async def handler(request: web.Request) -> web.Response:
+            nonlocal t
+            t = asyncio.create_task(task())
+            return web.Response(text="FOO")
+
+        t = test_task = None
+        app = web.Application()
+        app.cleanup_ctx.append(run_test)
+        app.router.add_get("/", handler)
+        app.router.add_get("/stop", self.stop)
+
+        web.run_app(app, port=port, shutdown_timeout=timeout)
+        assert test_task.exception() is None
+        return t
+
+    def test_shutdown_wait_for_task(
+        self, aiohttp_unused_port: Callable[[], int]
+    ) -> None:
+        port = aiohttp_unused_port()
+        finished = False
+
+        async def task():
+            nonlocal finished
+            await asyncio.sleep(2)
+            finished = True
+
+        t = self.run_app(port, 3, task)
+
+        assert finished is True
+        assert t.done()
+        assert not t.cancelled()
+
+    def test_shutdown_timeout_task(
+        self, aiohttp_unused_port: Callable[[], int]
+    ) -> None:
+        port = aiohttp_unused_port()
+        finished = False
+
+        async def task():
+            nonlocal finished
+            await asyncio.sleep(2)
+            finished = True
+
+        t = self.run_app(port, 1, task)
+
+        assert finished is False
+        assert t.done()
+        assert t.cancelled()
+
+    def test_shutdown_wait_for_spawned_task(
+        self, aiohttp_unused_port: Callable[[], int]
+    ) -> None:
+        port = aiohttp_unused_port()
+        finished = False
+        finished_sub = False
+        sub_t = None
+
+        async def sub_task():
+            nonlocal finished_sub
+            await asyncio.sleep(1.5)
+            finished_sub = True
+
+        async def task():
+            nonlocal finished, sub_t
+            await asyncio.sleep(0.5)
+            sub_t = asyncio.create_task(sub_task())
+            finished = True
+
+        t = self.run_app(port, 3, task)
+
+        assert finished is True
+        assert t.done()
+        assert not t.cancelled()
+        assert finished_sub is True
+        assert sub_t.done()
+        assert not sub_t.cancelled()
+
+    def test_shutdown_timeout_not_reached(
+        self, aiohttp_unused_port: Callable[[], int]
+    ) -> None:
+        port = aiohttp_unused_port()
+        finished = False
+
+        async def task():
+            nonlocal finished
+            await asyncio.sleep(1)
+            finished = True
+
+        start_time = time.time()
+        t = self.run_app(port, 15, task)
+
+        assert finished is True
+        assert t.done()
+        # Verify run_app has not waited for timeout.
+        assert time.time() - start_time < 10
+
+    def test_shutdown_new_conn_rejected(
+        self, aiohttp_unused_port: Callable[[], int]
+    ) -> None:
+        port = aiohttp_unused_port()
+        finished = False
+
+        async def task() -> None:
+            nonlocal finished
+            await asyncio.sleep(9)
+            finished = True
+
+        async def test(sess: ClientSession) -> None:
+            # Ensure we are in the middle of shutdown (waiting for task()).
+            await asyncio.sleep(1)
+            with pytest.raises(ClientConnectorError):
+                # Use a new session to try and open a new connection.
+                async with ClientSession() as sess:
+                    async with sess.get(f"http://localhost:{port}/"):
+                        pass
+            assert finished is False
+
+        t = self.run_app(port, 10, task, test)
+
+        assert finished is True
+        assert t.done()
+
+    def test_shutdown_pending_handler_responds(
+        self, aiohttp_unused_port: Callable[[], int]
+    ) -> None:
+        port = aiohttp_unused_port()
+        finished = False
+
+        async def test() -> None:
+            async def test_resp(sess):
+                async with sess.get(f"http://localhost:{port}/") as resp:
+                    assert await resp.text() == "FOO"
+
+            await asyncio.sleep(1)
+            async with ClientSession() as sess:
+                t = asyncio.create_task(test_resp(sess))
+                await asyncio.sleep(1)
+                # Handler is in-progress while we trigger server shutdown.
+                async with sess.get(f"http://localhost:{port}/stop"):
+                    pass
+
+                assert finished is False
+                # Handler should still complete and produce a response.
+                await t
+
+        async def run_test(app: web.Application) -> None:
+            nonlocal t
+            t = asyncio.create_task(test())
+            yield
+            await t
+
+        async def handler(request: web.Request) -> web.Response:
+            nonlocal finished
+            await asyncio.sleep(3)
+            finished = True
+            return web.Response(text="FOO")
+
+        t = None
+        app = web.Application()
+        app.cleanup_ctx.append(run_test)
+        app.router.add_get("/", handler)
+        app.router.add_get("/stop", self.stop)
+
+        web.run_app(app, port=port, shutdown_timeout=5)
+        assert t.exception() is None
+        assert finished is True
+
+    def test_shutdown_close_idle_keepalive(
+        self, aiohttp_unused_port: Callable[[], int]
+    ) -> None:
+        port = aiohttp_unused_port()
+
+        async def test() -> None:
+            await asyncio.sleep(1)
+            async with ClientSession() as sess:
+                async with sess.get(f"http://localhost:{port}/stop"):
+                    pass
+
+                # Hold on to keep-alive connection.
+                await asyncio.sleep(5)
+
+        async def run_test(app: web.Application) -> None:
+            nonlocal t
+            t = asyncio.create_task(test())
+            yield
+            t.cancel()
+            with contextlib.suppress(asyncio.CancelledError):
+                await t
+
+        t = None
+        app = web.Application()
+        app.cleanup_ctx.append(run_test)
+        app.router.add_get("/stop", self.stop)
+
+        web.run_app(app, port=port, shutdown_timeout=10)
+        # If connection closed, then test() will be cancelled in cleanup_ctx.
+        # If not, then shutdown_timeout will allow it to sleep until complete.
+        assert t.cancelled()
+
+    def test_shutdown_close_websockets(
+        self, aiohttp_unused_port: Callable[[], int]
+    ) -> None:
+        port = aiohttp_unused_port()
+        WS = web.AppKey("ws", Set[web.WebSocketResponse])
+        client_finished = server_finished = False
+
+        async def ws_handler(request: web.Request) -> web.WebSocketResponse:
+            ws = web.WebSocketResponse()
+            await ws.prepare(request)
+            request.app[WS].add(ws)
+            async for msg in ws:
+                pass
+            nonlocal server_finished
+            server_finished = True
+            return ws
+
+        async def close_websockets(app: web.Application) -> None:
+            for ws in app[WS]:
+                await ws.close(code=WSCloseCode.GOING_AWAY)
+
+        async def test() -> None:
+            await asyncio.sleep(1)
+            async with ClientSession() as sess:
+                async with sess.ws_connect(f"http://localhost:{port}/ws") as ws:
+                    async with sess.get(f"http://localhost:{port}/stop"):
+                        pass
+
+                    async for msg in ws:
+                        pass
+                    nonlocal client_finished
+                    client_finished = True
+
+        async def run_test(app: web.Application) -> None:
+            nonlocal t
+            t = asyncio.create_task(test())
+            yield
+            t.cancel()
+            with contextlib.suppress(asyncio.CancelledError):
+                await t
+
+        t = None
+        app = web.Application()
+        app[WS] = set()
+        app.on_shutdown.append(close_websockets)
+        app.cleanup_ctx.append(run_test)
+        app.router.add_get("/ws", ws_handler)
+        app.router.add_get("/stop", self.stop)
+
+        start = time.time()
+        web.run_app(app, port=port, shutdown_timeout=10)
+        assert time.time() - start < 5
+        assert client_finished
+        assert server_finished
