@@ -4,6 +4,7 @@ import json
 import pathlib
 import socket
 import zlib
+from typing import Any, Optional
 from unittest import mock
 
 import pytest
@@ -11,11 +12,24 @@ from multidict import CIMultiDictProxy, MultiDict
 from yarl import URL
 
 import aiohttp
-from aiohttp import FormData, HttpVersion10, HttpVersion11, TraceConfig, multipart, web
+from aiohttp import (
+    FormData,
+    HttpVersion,
+    HttpVersion10,
+    HttpVersion11,
+    TraceConfig,
+    multipart,
+    web,
+)
 from aiohttp.hdrs import CONTENT_LENGTH, CONTENT_TYPE, TRANSFER_ENCODING
 from aiohttp.test_utils import make_mocked_coro
 from aiohttp.typedefs import Handler
 
+try:
+    import brotlicffi as brotli
+except ImportError:
+    import brotli
+
 try:
     import ssl
 except ImportError:
@@ -127,9 +141,13 @@ async def test_head_returns_empty_body(aiohttp_client) -> None:
     assert 200 == resp.status
     txt = await resp.text()
     assert "" == txt
+    # The Content-Length header should be set to 4 which is
+    # the length of the response body if it would have been
+    # returned by a GET request.
+    assert resp.headers["Content-Length"] == "4"
 
 
-async def test_response_before_complete(aiohttp_client) -> None:
+async def test_response_before_complete(aiohttp_client: Any) -> None:
     async def handler(request):
         return web.Response(body=b"OK")
 
@@ -571,6 +589,32 @@ async def test_100_continue_custom_response(aiohttp_client) -> None:
     await resp.release()
 
 
+async def test_expect_handler_custom_response(aiohttp_client) -> None:
+    cache = {"foo": "bar"}
+
+    async def handler(request: web.Request) -> web.Response:
+        return web.Response(text="handler")
+
+    async def expect_handler(request: web.Request) -> Optional[web.Response]:
+        k = request.headers.get("X-Key")
+        cached_value = cache.get(k)
+        if cached_value:
+            return web.Response(text=cached_value)
+
+    app = web.Application()
+    # expect_handler is only typed on add_route().
+    app.router.add_route("POST", "/", handler, expect_handler=expect_handler)
+    client = await aiohttp_client(app)
+
+    async with client.post("/", expect100=True, headers={"X-Key": "foo"}) as resp:
+        assert resp.status == 200
+        assert await resp.text() == "bar"
+
+    async with client.post("/", expect100=True, headers={"X-Key": "spam"}) as resp:
+        assert resp.status == 200
+        assert await resp.text() == "handler"
+
+
 async def test_100_continue_for_not_found(aiohttp_client) -> None:
 
     app = web.Application()
@@ -1051,53 +1095,22 @@ async def test_response_with_payload_stringio(aiohttp_client, fname) -> None:
     await resp.release()
 
 
-async def test_response_with_precompressed_body_gzip(aiohttp_client) -> None:
-    async def handler(request):
-        headers = {"Content-Encoding": "gzip"}
-        zcomp = zlib.compressobj(wbits=16 + zlib.MAX_WBITS)
-        data = zcomp.compress(b"mydata") + zcomp.flush()
-        return web.Response(body=data, headers=headers)
-
-    app = web.Application()
-    app.router.add_get("/", handler)
-    client = await aiohttp_client(app)
-
-    resp = await client.get("/")
-    assert 200 == resp.status
-    data = await resp.read()
-    assert b"mydata" == data
-    assert resp.headers.get("Content-Encoding") == "gzip"
-
-    await resp.release()
-
-
-async def test_response_with_precompressed_body_deflate(aiohttp_client) -> None:
-    async def handler(request):
-        headers = {"Content-Encoding": "deflate"}
-        zcomp = zlib.compressobj(wbits=zlib.MAX_WBITS)
-        data = zcomp.compress(b"mydata") + zcomp.flush()
-        return web.Response(body=data, headers=headers)
-
-    app = web.Application()
-    app.router.add_get("/", handler)
-    client = await aiohttp_client(app)
-
-    resp = await client.get("/")
-    assert 200 == resp.status
-    data = await resp.read()
-    assert b"mydata" == data
-    assert resp.headers.get("Content-Encoding") == "deflate"
-
-    await resp.release()
-
-
-async def test_response_with_precompressed_body_deflate_no_hdrs(aiohttp_client) -> None:
-    async def handler(request):
-        headers = {"Content-Encoding": "deflate"}
+@pytest.mark.parametrize(
+    "compressor,encoding",
+    [
+        (zlib.compressobj(wbits=16 + zlib.MAX_WBITS), "gzip"),
+        (zlib.compressobj(wbits=zlib.MAX_WBITS), "deflate"),
         # Actually, wrong compression format, but
         # should be supported for some legacy cases.
-        zcomp = zlib.compressobj(wbits=-zlib.MAX_WBITS)
-        data = zcomp.compress(b"mydata") + zcomp.flush()
+        (zlib.compressobj(wbits=-zlib.MAX_WBITS), "deflate"),
+    ],
+)
+async def test_response_with_precompressed_body(
+    aiohttp_client, compressor, encoding
+) -> None:
+    async def handler(request):
+        headers = {"Content-Encoding": encoding}
+        data = compressor.compress(b"mydata") + compressor.flush()
         return web.Response(body=data, headers=headers)
 
     app = web.Application()
@@ -1108,7 +1121,27 @@ async def test_response_with_precompressed_body_deflate_no_hdrs(aiohttp_client)
     assert 200 == resp.status
     data = await resp.read()
     assert b"mydata" == data
-    assert resp.headers.get("Content-Encoding") == "deflate"
+    assert resp.headers.get("Content-Encoding") == encoding
+
+    await resp.release()
+
+
+async def test_response_with_precompressed_body_brotli(aiohttp_client) -> None:
+    async def handler(request):
+        headers = {"Content-Encoding": "br"}
+        return web.Response(body=brotli.compress(b"mydata"), headers=headers)
+
+    app = web.Application()
+    app.router.add_get("/", handler)
+    client = await aiohttp_client(app)
+
+    resp = await client.get("/")
+    assert 200 == resp.status
+    data = await resp.read()
+    assert b"mydata" == data
+    assert resp.headers.get("Content-Encoding") == "br"
+
+    await resp.release()
 
 
 async def test_bad_request_payload(aiohttp_client) -> None:
@@ -1534,26 +1567,27 @@ async def test_subapp_middleware_context(aiohttp_client, route, expected, middle
     def show_app_context(appname):
         @web.middleware
         async def middleware(request, handler: Handler):
-            values.append("{}: {}".format(appname, request.app["my_value"]))
+            values.append(f"{appname}: {request.app[my_value]}")
             return await handler(request)
 
         return middleware
 
     def make_handler(appname):
         async def handler(request):
-            values.append("{}: {}".format(appname, request.app["my_value"]))
+            values.append(f"{appname}: {request.app[my_value]}")
             return web.Response(text="Ok")
 
         return handler
 
     app = web.Application()
-    app["my_value"] = "root"
+    my_value = web.AppKey("my_value", str)
+    app[my_value] = "root"
     if "A" in middlewares:
         app.middlewares.append(show_app_context("A"))
     app.router.add_get("/", make_handler("B"))
 
     subapp = web.Application()
-    subapp["my_value"] = "sub"
+    subapp[my_value] = "sub"
     if "C" in middlewares:
         subapp.middlewares.append(show_app_context("C"))
     subapp.router.add_get("/", make_handler("D"))
@@ -1821,7 +1855,9 @@ async def test_await(aiohttp_server) -> None:
     async def handler(request):
         resp = web.StreamResponse(headers={"content-length": str(4)})
         await resp.prepare(request)
-        with pytest.warns(DeprecationWarning):
+        with pytest.deprecated_call(
+            match=r"^drain method is deprecated, use await resp\.write\(\)$",
+        ):
             await resp.drain()
         await asyncio.sleep(0.01)
         await resp.write(b"test")
@@ -1853,7 +1889,6 @@ async def test_response_context_manager(aiohttp_server) -> None:
     resp = await session.get(server.make_url("/"))
     async with resp:
         assert resp.status == 200
-        assert resp.connection is None
     assert resp.connection is None
 
     await session.close()
@@ -1900,7 +1935,9 @@ async def test_context_manager_close_on_release(aiohttp_server, mocker) -> None:
     async def handler(request):
         resp = web.StreamResponse()
         await resp.prepare(request)
-        with pytest.warns(DeprecationWarning):
+        with pytest.deprecated_call(
+            match=r"^drain method is deprecated, use await resp\.write\(\)$",
+        ):
             await resp.drain()
         await asyncio.sleep(10)
         return resp
@@ -1919,6 +1956,8 @@ async def test_context_manager_close_on_release(aiohttp_server, mocker) -> None:
         assert resp.connection is None
         assert proto.close.called
 
+        await resp.release()  # Trigger handler completion
+
 
 async def test_iter_any(aiohttp_server) -> None:
 
@@ -2169,3 +2208,38 @@ async def test_stream_response_headers_204(aiohttp_client):
     assert CONTENT_TYPE not in resp.headers
     assert TRANSFER_ENCODING not in resp.headers
     await resp.release()
+
+
+async def test_httpfound_cookies_302(aiohttp_client: Any) -> None:
+    async def handler(_):
+        resp = web.HTTPFound("/")
+        resp.set_cookie("my-cookie", "cookie-value")
+        raise resp
+
+    app = web.Application()
+    app.router.add_get("/", handler)
+    client = await aiohttp_client(app)
+
+    resp = await client.get("/", allow_redirects=False)
+    assert "my-cookie" in resp.cookies
+    await resp.release()
+
+
+@pytest.mark.parametrize("status", (101, 204, 304))
+@pytest.mark.parametrize("version", (HttpVersion10, HttpVersion11))
+async def test_no_body_for_1xx_204_304_responses(
+    aiohttp_client: Any, status: int, version: HttpVersion
+) -> None:
+    """Test no body is present for for 1xx, 204, and 304 responses."""
+
+    async def handler(_):
+        return web.Response(status=status, body=b"should not get to client")
+
+    app = web.Application()
+    app.router.add_get("/", handler)
+    client = await aiohttp_client(app, version=version)
+    resp = await client.get("/")
+    assert CONTENT_TYPE not in resp.headers
+    assert TRANSFER_ENCODING not in resp.headers
+    await resp.read() == b""
+    await resp.release()
