@@ -1,3 +1,4 @@
+from pathlib import Path
 from unittest import mock
 
 from aiohttp import hdrs
@@ -10,19 +11,17 @@ def test_using_gzip_if_header_present_and_file_available(loop) -> None:
         "GET", "http://python.org/logo.png", headers={hdrs.ACCEPT_ENCODING: "gzip"}
     )
 
-    gz_filepath = mock.Mock()
-    gz_filepath.open = mock.mock_open()
+    gz_filepath = mock.create_autospec(Path, spec_set=True)
     gz_filepath.is_file.return_value = True
-    gz_filepath.stat.return_value = mock.MagicMock()
     gz_filepath.stat.return_value.st_size = 1024
     gz_filepath.stat.return_value.st_mtime_ns = 1603733507222449291
 
-    filepath = mock.Mock()
+    filepath = mock.create_autospec(Path, spec_set=True)
     filepath.name = "logo.png"
-    filepath.open = mock.mock_open()
     filepath.with_name.return_value = gz_filepath
 
     file_sender = FileResponse(filepath)
+    file_sender._path = filepath
     file_sender._sendfile = make_mocked_coro(None)  # type: ignore[assignment]
 
     loop.run_until_complete(file_sender.prepare(request))
@@ -34,19 +33,17 @@ def test_using_gzip_if_header_present_and_file_available(loop) -> None:
 def test_gzip_if_header_not_present_and_file_available(loop) -> None:
     request = make_mocked_request("GET", "http://python.org/logo.png", headers={})
 
-    gz_filepath = mock.Mock()
-    gz_filepath.open = mock.mock_open()
+    gz_filepath = mock.create_autospec(Path, spec_set=True)
     gz_filepath.is_file.return_value = True
 
-    filepath = mock.Mock()
+    filepath = mock.create_autospec(Path, spec_set=True)
     filepath.name = "logo.png"
-    filepath.open = mock.mock_open()
     filepath.with_name.return_value = gz_filepath
-    filepath.stat.return_value = mock.MagicMock()
     filepath.stat.return_value.st_size = 1024
     filepath.stat.return_value.st_mtime_ns = 1603733507222449291
 
     file_sender = FileResponse(filepath)
+    file_sender._path = filepath
     file_sender._sendfile = make_mocked_coro(None)  # type: ignore[assignment]
 
     loop.run_until_complete(file_sender.prepare(request))
@@ -58,19 +55,17 @@ def test_gzip_if_header_not_present_and_file_available(loop) -> None:
 def test_gzip_if_header_not_present_and_file_not_available(loop) -> None:
     request = make_mocked_request("GET", "http://python.org/logo.png", headers={})
 
-    gz_filepath = mock.Mock()
-    gz_filepath.open = mock.mock_open()
+    gz_filepath = mock.create_autospec(Path, spec_set=True)
     gz_filepath.is_file.return_value = False
 
-    filepath = mock.Mock()
+    filepath = mock.create_autospec(Path, spec_set=True)
     filepath.name = "logo.png"
-    filepath.open = mock.mock_open()
     filepath.with_name.return_value = gz_filepath
-    filepath.stat.return_value = mock.MagicMock()
     filepath.stat.return_value.st_size = 1024
     filepath.stat.return_value.st_mtime_ns = 1603733507222449291
 
     file_sender = FileResponse(filepath)
+    file_sender._path = filepath
     file_sender._sendfile = make_mocked_coro(None)  # type: ignore[assignment]
 
     loop.run_until_complete(file_sender.prepare(request))
@@ -84,19 +79,17 @@ def test_gzip_if_header_present_and_file_not_available(loop) -> None:
         "GET", "http://python.org/logo.png", headers={hdrs.ACCEPT_ENCODING: "gzip"}
     )
 
-    gz_filepath = mock.Mock()
-    gz_filepath.open = mock.mock_open()
+    gz_filepath = mock.create_autospec(Path, spec_set=True)
     gz_filepath.is_file.return_value = False
 
-    filepath = mock.Mock()
+    filepath = mock.create_autospec(Path, spec_set=True)
     filepath.name = "logo.png"
-    filepath.open = mock.mock_open()
     filepath.with_name.return_value = gz_filepath
-    filepath.stat.return_value = mock.MagicMock()
     filepath.stat.return_value.st_size = 1024
     filepath.stat.return_value.st_mtime_ns = 1603733507222449291
 
     file_sender = FileResponse(filepath)
+    file_sender._path = filepath
     file_sender._sendfile = make_mocked_coro(None)  # type: ignore[assignment]
 
     loop.run_until_complete(file_sender.prepare(request))
@@ -108,14 +101,13 @@ def test_gzip_if_header_present_and_file_not_available(loop) -> None:
 def test_status_controlled_by_user(loop) -> None:
     request = make_mocked_request("GET", "http://python.org/logo.png", headers={})
 
-    filepath = mock.Mock()
+    filepath = mock.create_autospec(Path, spec_set=True)
     filepath.name = "logo.png"
-    filepath.open = mock.mock_open()
-    filepath.stat.return_value = mock.MagicMock()
     filepath.stat.return_value.st_size = 1024
     filepath.stat.return_value.st_mtime_ns = 1603733507222449291
 
     file_sender = FileResponse(filepath, status=203)
+    file_sender._path = filepath
     file_sender._sendfile = make_mocked_coro(None)  # type: ignore[assignment]
 
     loop.run_until_complete(file_sender.prepare(request))
